@@ -1,5 +1,5 @@
 /***************************************************************************
-    copyright            : (C) 2003-2005 by Robby Stephenson
+    copyright            : (C) 2003-2006 by Robby Stephenson
     email                : robby@periapsis.org
  ***************************************************************************/
 
@@ -12,14 +12,11 @@
  ***************************************************************************/
 
 #include "bibtexcollection.h"
-#include "../collectionfactory.h"
 #include "../latin1literal.h"
-#include "../translators/bibtexhandler.h"
+#include "../document.h"
+#include "../tellico_debug.h"
 
 #include <klocale.h>
-#include <kdebug.h>
-
-#include <qfile.h>
 
 using Tellico::Data::BibtexCollection;
 
@@ -30,7 +27,7 @@ namespace {
 }
 
 BibtexCollection::BibtexCollection(bool addFields_, const QString& title_ /*=null*/)
-   : Collection(title_, CollectionFactory::entryName(Bibtex), i18n("Entries")) {
+   : Collection(title_, i18n("Entries")) {
   setTitle(title_.isNull() ? i18n("Bibliography") : title_);
   if(addFields_) {
     addFields(defaultFields());
@@ -54,7 +51,7 @@ BibtexCollection::BibtexCollection(bool addFields_, const QString& title_ /*=nul
 
 Tellico::Data::FieldVec BibtexCollection::defaultFields() {
   FieldVec list;
-  Field* field;
+  FieldPtr field;
 
 /******************* General ****************************/
   field = new Field(QString::fromLatin1("title"), i18n("Title"));
@@ -152,11 +149,11 @@ Tellico::Data::FieldVec BibtexCollection::defaultFields() {
   field->setFlags(Field::AllowGrouped);
   list.append(field);
 
-//  field = new Field(QString::fromLatin1("isbn"), i18n("ISBN#"));
-//  field->setProperty(QString::fromLatin1("bibtex"), QString::fromLatin1("isbn"));
-//  field->setCategory(i18n(bibtex_publishing));
-//  field->setDescription(i18n("International Standard Book Number"));
-//  list.append(field);
+  field = new Field(QString::fromLatin1("isbn"), i18n("ISBN#"));
+  field->setProperty(QString::fromLatin1("bibtex"), QString::fromLatin1("isbn"));
+  field->setCategory(i18n(bibtex_publishing));
+  field->setDescription(i18n("International Standard Book Number"));
+  list.append(field);
 
   field = new Field(QString::fromLatin1("journal"), i18n("Journal"));
   field->setProperty(QString::fromLatin1("bibtex"), QString::fromLatin1("journal"));
@@ -167,13 +164,13 @@ Tellico::Data::FieldVec BibtexCollection::defaultFields() {
 
   // could make this a string list, but since bibtex import could have funky values
   // keep it an editbox
-  field = new Field(QString::fromLatin1("month"), i18n("Month"), Field::Number);
+  field = new Field(QString::fromLatin1("month"), i18n("Month"));
   field->setProperty(QString::fromLatin1("bibtex"), QString::fromLatin1("month"));
   field->setCategory(i18n(bibtex_publishing));
   field->setFlags(Field::AllowCompletion);
   list.append(field);
 
-  field = new Field(QString::fromLatin1("number"), i18n("Number"));
+  field = new Field(QString::fromLatin1("number"), i18n("Number"), Field::Number);
   field->setProperty(QString::fromLatin1("bibtex"), QString::fromLatin1("number"));
   field->setCategory(i18n(bibtex_publishing));
   list.append(field);
@@ -217,19 +214,27 @@ Tellico::Data::FieldVec BibtexCollection::defaultFields() {
 //  field->setCategory(i18n(bibtex_misc));
 //  list.append(field);
 
-  field = new Field(QString::fromLatin1("note"), i18n("Notes"));
-  field->setProperty(QString::fromLatin1("bibtex"), QString::fromLatin1("note"));
+  field = new Field(QString::fromLatin1("keyword"), i18n("Keywords"));
   field->setCategory(i18n(bibtex_misc));
+  field->setFlags(Field::AllowCompletion | Field::AllowMultiple | Field::AllowGrouped);
+  list.append(field);
+
+  field = new Field(QString::fromLatin1("abstract"), i18n("Abstract"), Data::Field::Para);
+  field->setProperty(QString::fromLatin1("bibtex"), QString::fromLatin1("abstract"));
+  list.append(field);
+
+  field = new Field(QString::fromLatin1("note"), i18n("Notes"), Field::Para);
+  field->setProperty(QString::fromLatin1("bibtex"), QString::fromLatin1("note"));
   list.append(field);
 
   return list;
 }
 
-bool BibtexCollection::addField(Field* field_) {
+bool BibtexCollection::addField(FieldPtr field_) {
   if(!field_) {
     return false;
   }
-//  kdDebug() << "BibtexCollection::addField()" << endl;
+//  myDebug() << "BibtexCollection::addField()" << endl;
   bool success = Collection::addField(field_);
   if(success) {
     QString bibtex = field_->property(QString::fromLatin1("bibtex"));
@@ -240,13 +245,13 @@ bool BibtexCollection::addField(Field* field_) {
   return success;
 }
 
-bool BibtexCollection::modifyField(Field* newField_) {
+bool BibtexCollection::modifyField(FieldPtr newField_) {
   if(!newField_) {
     return false;
   }
-//  kdDebug() << "BibtexCollection::modifyField()" << endl;
+//  myDebug() << "BibtexCollection::modifyField()" << endl;
   bool success = Collection::modifyField(newField_);
-  Field* oldField = fieldByName(newField_->name());
+  FieldPtr oldField = fieldByName(newField_->name());
   QString oldBibtex = oldField->property(QString::fromLatin1("bibtex"));
   QString newBibtex = newField_->property(QString::fromLatin1("bibtex"));
   if(!oldBibtex.isEmpty()) {
@@ -259,11 +264,11 @@ bool BibtexCollection::modifyField(Field* newField_) {
   return success;
 }
 
-bool BibtexCollection::deleteField(Field* field_, bool force_) {
+bool BibtexCollection::deleteField(FieldPtr field_, bool force_) {
   if(!field_) {
     return false;
   }
-//  kdDebug() << "BibtexCollection::deleteField()" << endl;
+//  myDebug() << "BibtexCollection::deleteField()" << endl;
   bool success = true;
   QString bibtex = field_->property(QString::fromLatin1("bibtex"));
   if(!bibtex.isEmpty()) {
@@ -272,19 +277,45 @@ bool BibtexCollection::deleteField(Field* field_, bool force_) {
   return success && Collection::removeField(field_, force_);
 }
 
-Tellico::Data::Field* const BibtexCollection::fieldByBibtexName(const QString& bibtex_) const {
+Tellico::Data::FieldPtr BibtexCollection::fieldByBibtexName(const QString& bibtex_) const {
   return m_bibtexFieldDict.isEmpty() ? 0 : m_bibtexFieldDict.find(bibtex_);
 }
 
+// same as BookCollection::SameEntry()
+int BibtexCollection::sameEntry(Data::EntryPtr entry1_, Data::EntryPtr entry2_) const {
+  // equal isbn's or lccn's are easy, give it a weight of 100
+  if(Entry::compareValues(entry1_, entry2_, QString::fromLatin1("isbn"), this) > 0 ||
+     Entry::compareValues(entry1_, entry2_, QString::fromLatin1("lccn"), this) > 0) {
+    return 100; // good match
+  }
+  int res = 3*Entry::compareValues(entry1_, entry2_, QString::fromLatin1("title"), this);
+//  if(res == 0) {
+//    myDebug() << "BookCollection::sameEntry() - different titles for " << entry1_->title() << " vs. "
+//              << entry2_->title() << endl;
+//  }
+  res += Entry::compareValues(entry1_, entry2_, QString::fromLatin1("author"), this);
+  res += Entry::compareValues(entry1_, entry2_, QString::fromLatin1("cr_year"), this);
+  res += Entry::compareValues(entry1_, entry2_, QString::fromLatin1("pub_year"), this);
+  res += Entry::compareValues(entry1_, entry2_, QString::fromLatin1("binding"), this);
+  return res;
+}
+
 // static
-BibtexCollection* BibtexCollection::convertBookCollection(const Collection* coll_) {
+Tellico::Data::CollPtr BibtexCollection::convertBookCollection(CollPtr coll_) {
   const QString bibtex = QString::fromLatin1("bibtex");
-  BibtexCollection* coll = new BibtexCollection(false, coll_->title());
+  KSharedPtr<BibtexCollection> coll = new BibtexCollection(false, coll_->title());
   FieldVec fields = coll_->fields();
   for(FieldVec::Iterator fIt = fields.begin(); fIt != fields.end(); ++fIt) {
-    Field* field = fIt->clone();
-    QString name = field->name();
+    FieldPtr field = new Data::Field(*fIt);
 
+    // if it already has a bibtex property, skip it
+    if(!field->property(bibtex).isEmpty()) {
+      coll->addField(field);
+      continue;
+    }
+
+    // be sure to set bibtex property before adding it though
+    QString name = field->name();
     // this first group has bibtex field names the same as their own field name
     if(name == Latin1Literal("title")
        || name == Latin1Literal("author")
@@ -317,62 +348,36 @@ BibtexCollection* BibtexCollection::convertBookCollection(const Collection* coll
   // also need to add required fields
   FieldVec vec = defaultFields();
   for(FieldVec::Iterator it = vec.begin(); it != vec.end(); ++it) {
-    if(coll->fieldByName(it->name()) == 0 && (it->flags() & Field::NoDelete)) {
+    if(!coll->hasField(it->name()) && (it->flags() & Field::NoDelete)) {
       // but don't add a Bibtex Key if there's already a bibtex-id
       if(it->property(bibtex) != Latin1Literal("key")
-         || coll->fieldByName(QString::fromLatin1("bibtex-id")) == 0) {
+         || !coll->hasField(QString::fromLatin1("bibtex-id"))) {
         coll->addField(it);
       }
     }
   }
 
   // set the entry-type to book
-  Field* field = coll->fieldByBibtexName(QString::fromLatin1("entry-type"));
+  FieldPtr field = coll->fieldByBibtexName(QString::fromLatin1("entry-type"));
   QString entryTypeName;
   if(field) {
     entryTypeName = field->name();
+  } else {
+    kdWarning() << "BibtexCollection::convertBookCollection() - there must be an entry type field" << endl;
   }
 
   for(EntryVec::ConstIterator entryIt = coll_->entries().begin(); entryIt != coll_->entries().end(); ++entryIt) {
-    Data::Entry* entry = new Entry(*entryIt, coll);
+    Data::EntryPtr entry = new Entry(*entryIt);
     coll->addEntry(entry);
     if(!entryTypeName.isEmpty()) {
       entry->setField(entryTypeName, QString::fromLatin1("book"));
     }
   }
 
-  return coll;
-}
+  // now need to make sure all images are transferred
+  Document::self()->loadAllImagesNow();
 
-QStringList BibtexCollection::bibtexKeys(const EntryVec& entries_) const {
-  QStringList keys;
-  for(Data::EntryVec::ConstIterator it = entries_.begin(); it != entries_.end(); ++it) {
-    keys << BibtexHandler::bibtexKey(it);
-  }
-  return keys;
-}
-
-void BibtexCollection::citeEntries(QFile& lyxpipe_, const EntryVec& entries_) const {
-  if(entries_.isEmpty()) {
-    return;
-  }
-//  kdDebug() << "BibtexCollection::citeEntries()" << endl;
-  const QStringList keys = bibtexKeys(entries_);
-
-  QTextStream ts(&lyxpipe_);
-//  ts << "LYXSRV:tellico:hello\n";
-  ts << "LYXCMD:tellico:citation-insert:";
-  QStringList::ConstIterator end = keys.end();
-  for(QStringList::ConstIterator it = keys.begin(); it != end; ++it) {
-    ts << (*it).latin1();
-    if(it != keys.fromLast()) {
-      // pybliographer uses comma-space, and pyblink expects the space there
-      ts << ", ";
-    }
-  }
-  ts << "\n";
-//  ts << "LYXSRV:tellico:bye\n";
-  lyxpipe_.flush();
+  return coll.data();
 }
 
 #include "bibtexcollection.moc"
