@@ -1,5 +1,5 @@
 /***************************************************************************
-    copyright            : (C) 2005 by Robby Stephenson
+    copyright            : (C) 2005-2006 by Robby Stephenson
     email                : robby@periapsis.org
  ***************************************************************************/
 
@@ -33,7 +33,7 @@ public:
     FilterRemove
   };
 
-  FilterCommand(Mode mode, Filter* activeFilter, Filter* oldFilter=0);
+  FilterCommand(Mode mode, FilterPtr activeFilter, FilterPtr oldFilter=0);
 
   virtual void execute();
   virtual void unexecute();
