@@ -1,5 +1,5 @@
 /***************************************************************************
-    copyright            : (C) 2003-2005 by Robby Stephenson
+    copyright            : (C) 2003-2006 by Robby Stephenson
     email                : robby@periapsis.org
  ***************************************************************************/
 
@@ -36,11 +36,12 @@
 #include "tellico_debug.h"
 #include "groupiterator.h"
 #include "tellico_utils.h"
+#include "entryupdater.h"
 
-#include <kdebug.h>
 #include <klocale.h>
 #include <kmessagebox.h>
 #include <kaction.h>
+#include <ktoolbarbutton.h>
 
 using Tellico::Controller;
 
@@ -48,7 +49,6 @@ Controller* Controller::s_self = 0;
 
 Controller::Controller(MainWindow* parent_, const char* name_)
     : QObject(parent_, name_), m_mainWindow(parent_), m_working (false) {
-  m_widgetBlocks.setAutoDelete(true);
 }
 
 void Controller::addObserver(Observer* obs) {
@@ -100,7 +100,7 @@ Tellico::Data::EntryVec Controller::visibleEntries() {
   return m_mainWindow->m_detailedView->visibleEntries();
 }
 
-void Controller::slotCollectionAdded(Data::Collection* coll_) {
+void Controller::slotCollectionAdded(Data::CollPtr coll_) {
 //  myDebug() << "Controller::slotCollectionAdded()" << endl;
   // at start-up, this might get called too early, so check and bail
   if(!m_mainWindow->m_groupView) {
@@ -109,14 +109,8 @@ void Controller::slotCollectionAdded(Data::Collection* coll_) {
 
   // do this first because the group view will need it later
   m_mainWindow->readCollectionOptions(coll_);
-
-  // these might take some time, change the status message
-  // the detailed view also has half the progress bar
-  // this slot gets called after the importer has loaded the collection
-  // so first bump it to 100% of the step, increase the step, then load
-  // the document view
-  m_mainWindow->slotUpdateFractionDone(1.0);
-  m_mainWindow->m_currentStep = m_mainWindow->m_maxSteps;
+  m_mainWindow->updateEntrySources(); // has to be called before all the addCollection()
+  // calls in the widgets since they may want menu updates
 
 //  blockAllSignals(true);
   m_mainWindow->m_detailedView->addCollection(coll_);
@@ -134,31 +128,29 @@ void Controller::slotCollectionAdded(Data::Collection* coll_) {
   }
 //  blockAllSignals(false);
 
-  // The importer doesn't ever finish the import, better to do it here
-  m_mainWindow->slotUpdateFractionDone(1.0);
-  m_mainWindow->m_currentStep = 1;
   m_mainWindow->slotStatusMsg(i18n("Ready."));
 
   m_selectedEntries.clear();
   m_mainWindow->slotEntryCount();
+
   updateActions();
 
-  connect(coll_, SIGNAL(signalGroupModified(Tellico::Data::Collection*, Tellico::Data::EntryGroup*)),
-          m_mainWindow->m_groupView, SLOT(slotModifyGroup(Tellico::Data::Collection*, Tellico::Data::EntryGroup*)));
+  connect(coll_, SIGNAL(signalGroupModified(Tellico::Data::CollPtr, Tellico::Data::EntryGroup*)),
+          m_mainWindow->m_groupView, SLOT(slotModifyGroup(Tellico::Data::CollPtr, Tellico::Data::EntryGroup*)));
 
-  connect(coll_, SIGNAL(signalRefreshField(Tellico::Data::Field*)),
-          this, SLOT(slotRefreshField(Tellico::Data::Field*)));
+  connect(coll_, SIGNAL(signalRefreshField(Tellico::Data::FieldPtr)),
+          this, SLOT(slotRefreshField(Tellico::Data::FieldPtr)));
 }
 
-void Controller::slotCollectionModified(Data::Collection* coll_) {
+void Controller::slotCollectionModified(Data::CollPtr coll_) {
   // easiest thing is to signal collection deleted, then added?
-  // FIXME: fixme Signals for delete collection and then added are yucky
+  // FIXME: Signals for delete collection and then added are yucky
   slotCollectionDeleted(coll_);
   slotCollectionAdded(coll_);
 }
 
-void Controller::slotCollectionDeleted(Data::Collection* coll_) {
-//  kdDebug() << "Controller::slotCollectionDeleted()" << endl;
+void Controller::slotCollectionDeleted(Data::CollPtr coll_) {
+//  myDebug() << "Controller::slotCollectionDeleted()" << endl;
 
   blockAllSignals(true);
   m_mainWindow->saveCollectionOptions(coll_);
@@ -173,24 +165,23 @@ void Controller::slotCollectionDeleted(Data::Collection* coll_) {
   m_mainWindow->m_viewStack->clear();
   blockAllSignals(false);
 
-  // disconnect all signals from the collection to the controller
+  // disconnect all signals from the collection
   // this is needed because the Collection::appendCollection() and mergeCollection()
   // functions signal collection deleted then added for the same collection
-  disconnect(coll_, 0, this, 0);
+  coll_->disconnect();
 }
 
 void Controller::addedEntries(Data::EntryVec entries_) {
   blockAllSignals(true);
-  for(Data::EntryVecIt entry = entries_.begin(); entry != entries_.end(); ++entry) {
-    for(ObserverVec::Iterator it = m_observers.begin(); it != m_observers.end(); ++it) {
-      it->addEntry(entry);
-    }
+  for(ObserverVec::Iterator it = m_observers.begin(); it != m_observers.end(); ++it) {
+    it->addEntries(entries_);
   }
+  m_mainWindow->slotQueueFilter();
   slotUpdateSelection(0, entries_);
   blockAllSignals(false);
 }
 
-void Controller::modifiedEntry(Data::Entry* entry_) {
+void Controller::modifiedEntries(Data::EntryVec entries_) {
   // when a new document is being loaded, loans are added to borrowers, which
   // end up calling Entry::checkIn() which called Document::saveEntry() which calls here
   // ignore that
@@ -199,55 +190,54 @@ void Controller::modifiedEntry(Data::Entry* entry_) {
   }
   blockAllSignals(true);
   for(ObserverVec::Iterator it = m_observers.begin(); it != m_observers.end(); ++it) {
-    it->modifyEntry(entry_);
+    it->modifyEntries(entries_);
   }
-  m_mainWindow->m_viewStack->refresh(); // special case
+  m_mainWindow->m_viewStack->entryView()->slotRefresh(); // special case
+  m_mainWindow->slotQueueFilter();
   blockAllSignals(false);
 }
 
 void Controller::removedEntries(Data::EntryVec entries_) {
-  blockAllUpdates(true);
   blockAllSignals(true);
-  for(Data::EntryVecIt entry = entries_.begin(); entry != entries_.end(); ++entry) {
-    for(ObserverVec::Iterator it = m_observers.begin(); it != m_observers.end(); ++it) {
-      it->removeEntry(entry);
-      if(!m_selectedEntries.isEmpty() && entry == m_selectedEntries[0]) {
-        m_mainWindow->m_viewStack->entryView()->clear();
-      }
-      m_selectedEntries.remove(entry);
-      m_currentEntries.remove(entry);
-    }
+  for(ObserverVec::Iterator it = m_observers.begin(); it != m_observers.end(); ++it) {
+    it->removeEntries(entries_);
+    m_mainWindow->m_viewStack->entryView()->clear();
+    m_selectedEntries.clear();
+    m_currentEntries.clear();
   }
+  m_mainWindow->slotQueueFilter();
   blockAllSignals(false);
-  blockAllUpdates(false);
 }
 
-void Controller::addedField(Data::Collection* coll_, Data::Field* field_) {
+void Controller::addedField(Data::CollPtr coll_, Data::FieldPtr field_) {
   for(ObserverVec::Iterator it = m_observers.begin(); it != m_observers.end(); ++it) {
     it->addField(coll_, field_);
   }
   m_mainWindow->m_viewStack->refresh();
   m_mainWindow->slotUpdateCollectionToolBar(coll_);
+  m_mainWindow->slotQueueFilter();
 }
 
-void Controller::removedField(Data::Collection* coll_, Data::Field* field_) {
-//  kdDebug() << "Controller::slotFieldDeleted() - " << field_->name() << endl;
+void Controller::removedField(Data::CollPtr coll_, Data::FieldPtr field_) {
+//  myDebug() << "Controller::slotFieldDeleted() - " << field_->name() << endl;
   for(ObserverVec::Iterator it = m_observers.begin(); it != m_observers.end(); ++it) {
     it->removeField(coll_, field_);
   }
   m_mainWindow->m_viewStack->refresh();
   m_mainWindow->slotUpdateCollectionToolBar(coll_);
+  m_mainWindow->slotQueueFilter();
 }
 
-void Controller::modifiedField(Data::Collection* coll_, Data::Field* oldField_, Data::Field* newField_) {
+void Controller::modifiedField(Data::CollPtr coll_, Data::FieldPtr oldField_, Data::FieldPtr newField_) {
   for(ObserverVec::Iterator it = m_observers.begin(); it != m_observers.end(); ++it) {
     it->modifyField(coll_, oldField_, newField_);
   }
   m_mainWindow->m_viewStack->refresh();
   m_mainWindow->slotUpdateCollectionToolBar(coll_);
+  m_mainWindow->slotQueueFilter();
 }
 
-void Controller::reorderedFields(Data::Collection* coll_) {
+void Controller::reorderedFields(Data::CollPtr coll_) {
   m_mainWindow->m_editDialog->setLayout(coll_);
   m_mainWindow->m_detailedView->reorderFields(coll_->fields());
   m_mainWindow->slotUpdateCollectionToolBar(coll_);
@@ -283,7 +273,7 @@ void Controller::slotUpdateSelection(QWidget* widget_, const Data::EntryVec& ent
     return;
   }
   m_working = true;
-//  kdDebug() << "Controller::slotUpdateSelection() entryList - " << list_.count() << endl;
+//  myDebug() << "Controller::slotUpdateSelection() entryList - " << list_.count() << endl;
 
   blockAllSignals(true);
 // in the list view and group view, if entries are selected in one, clear selection in other
@@ -318,6 +308,79 @@ void Controller::slotUpdateSelection(QWidget* widget_, const Data::EntryVec& ent
   m_working = false;
 }
 
+void Controller::slotGoPrevEntry() {
+  goEntrySibling(PrevEntry);
+}
+
+void Controller::slotGoNextEntry() {
+  goEntrySibling(NextEntry);
+}
+
+void Controller::goEntrySibling(EntryDirection dir_) {
+  // if there are currently multiple selected, then do nothing
+  if(m_selectedEntries.count() != 1) {
+    return;
+  }
+  // find the widget that has an entry selected
+  GUI::ListView* view = m_mainWindow->m_detailedView;
+  GUI::ListViewItemList items = view->selectedItems();
+  if(items.isEmpty()) {
+    view = m_mainWindow->m_groupView;
+    items = view->selectedItems();
+  }
+  if(items.isEmpty() && m_mainWindow->m_filterView) {
+    view = m_mainWindow->m_filterView;
+    items = view->selectedItems();
+  }
+  if(items.isEmpty() && m_mainWindow->m_loanView) {
+    view = m_mainWindow->m_loanView;
+    items = view->selectedItems();
+  }
+  if(items.count() != 1) {
+    return;
+  }
+  GUI::ListViewItem* item = items.first();
+  if(item->isEntryItem()) {
+    bool looped = false;
+    // check sanity
+    if(m_selectedEntries.front() != static_cast<EntryItem*>(item)->entry()) {
+      myDebug() << "Controller::slotGoNextEntry() - entries don't match!" << endl;
+    }
+    GUI::ListViewItem* nextItem = static_cast<GUI::ListViewItem*>(dir_ == PrevEntry
+                                                                         ? item->itemAbove()
+                                                                         : item->itemBelow());
+    if(!nextItem) {
+      // cycle through
+      nextItem = static_cast<GUI::ListViewItem*>(dir_ == PrevEntry
+                                                       ? view->lastItem()
+                                                       : view->firstChild());
+      looped = true;
+    }
+    while(nextItem && !nextItem->isEntryItem()) {
+      nextItem->setOpen(true); // have to be open to find the next one
+      nextItem = static_cast<GUI::ListViewItem*>(dir_ == PrevEntry
+                                                       ? nextItem->itemAbove()
+                                                       : nextItem->itemBelow());
+      if(!nextItem && !looped) {
+        // cycle through
+        nextItem = static_cast<GUI::ListViewItem*>(dir_ == PrevEntry
+                                                         ? view->lastItem()
+                                                         : view->firstChild());
+        looped = true;
+      }
+    }
+    if(nextItem) {
+      Data::EntryPtr e = static_cast<EntryItem*>(nextItem)->entry();
+      view->blockSignals(true);
+      view->setSelected(item, false);
+      view->setSelected(nextItem, true);
+      view->ensureItemVisible(nextItem);
+      view->blockSignals(false);
+      slotUpdateSelection(view, e);
+    }
+  }
+}
+
 void Controller::slotUpdateCurrent(const Data::EntryVec& entries_) {
   if(m_working) {
     return;
@@ -332,6 +395,20 @@ void Controller::slotUpdateCurrent(const Data::EntryVec& entries_) {
   m_working = false;
 }
 
+void Controller::slotUpdateSelectedEntries(const QString& source_) {
+  if(m_selectedEntries.isEmpty()) {
+    return;
+  }
+
+  // it deletes itself when done
+  // signal mapper strings can't be empty, "_all" is set in mainwindow
+  if(source_.isEmpty() || source_ == Latin1Literal("_all")) {
+    new EntryUpdater(m_selectedEntries.front()->collection(), m_selectedEntries, this);
+  } else {
+    new EntryUpdater(source_, m_selectedEntries.front()->collection(), m_selectedEntries, this);
+  }
+}
+
 void Controller::slotDeleteSelectedEntries() {
   if(m_selectedEntries.isEmpty()) {
     return;
@@ -377,8 +454,8 @@ void Controller::slotDeleteSelectedEntries() {
 //  slotUpdateSelection(m_mainWindow->m_detailedView, newList);
 }
 
-void Controller::slotRefreshField(Data::Field* field_) {
-  kdDebug() << "Controller::slotRefreshField()" << endl;
+void Controller::slotRefreshField(Data::FieldPtr field_) {
+//  myDebug() << "Controller::slotRefreshField()" << endl;
   // group view only needs to refresh if it's the title
   if(field_->name() == Latin1Literal("title")) {
     m_mainWindow->m_groupView->populateCollection();
@@ -400,7 +477,7 @@ void Controller::slotCopySelectedEntries() {
   for(Data::EntryVecIt it = m_selectedEntries.begin(); it != m_selectedEntries.end(); ++it) {
     entries.append(new Data::Entry(*it));
   }
-  Kernel::self()->saveEntries(Data::EntryVec(), entries);
+  Kernel::self()->addEntries(entries, false);
   slotUpdateSelection(0, old);
 }
 
@@ -421,30 +498,8 @@ void Controller::blockAllSignals(bool block_) const {
   m_mainWindow->m_viewStack->iconView()->blockSignals(block_);
 }
 
-void Controller::blockAllUpdates(bool block_) {
-  if(!m_mainWindow->m_initialized) {
-    return;
-  }
-
-  // if we don't want to block, clear and auto-delete all blocks
-  if(!block_) {
-    m_widgetBlocks.clear();
-    return;
-  }
-
-  m_widgetBlocks.append(new GUI::WidgetUpdateBlocker(m_mainWindow->m_detailedView));
-  m_widgetBlocks.append(new GUI::WidgetUpdateBlocker(m_mainWindow->m_groupView));
-  m_widgetBlocks.append(new GUI::WidgetUpdateBlocker(m_mainWindow->m_viewStack->iconView()));
-  if(m_mainWindow->m_loanView) {
-    m_widgetBlocks.append(new GUI::WidgetUpdateBlocker(m_mainWindow->m_loanView));
-  }
-  if(m_mainWindow->m_filterView) {
-    m_widgetBlocks.append(new GUI::WidgetUpdateBlocker(m_mainWindow->m_filterView));
-  }
-}
-
-void Controller::slotUpdateFilter(Filter* filter_) {
-//  kdDebug() << "Controller::slotUpdateFilter()" << endl;
+void Controller::slotUpdateFilter(FilterPtr filter_) {
+//  myDebug() << "Controller::slotUpdateFilter()" << endl;
   blockAllSignals(true);
 
   // the view takes over ownership of the filter
@@ -460,44 +515,99 @@ void Controller::slotUpdateFilter(Filter* filter_) {
   m_mainWindow->slotEntryCount();
 }
 
-void Controller::editEntry(const Data::Entry&) const {
+void Controller::editEntry(Data::EntryPtr) const {
   m_mainWindow->slotShowEntryEditor();
 }
 
-void Controller::plugCollectionActions(QWidget* widget_) {
-  m_mainWindow->action("coll_rename_collection")->plug(widget_);
-  m_mainWindow->action("coll_fields")->plug(widget_);
-  m_mainWindow->action("change_entry_grouping")->plug(widget_);
+void Controller::plugCollectionActions(QPopupMenu* popup_) {
+  if(!popup_) {
+    return;
+  }
+
+  m_mainWindow->action("coll_rename_collection")->plug(popup_);
+  m_mainWindow->action("coll_fields")->plug(popup_);
+  m_mainWindow->action("change_entry_grouping")->plug(popup_);
 }
 
-void Controller::plugEntryActions(QWidget* widget_) {
-//  m_mainWindow->m_newEntry->plug(widget_);
-  m_mainWindow->m_editEntry->plug(widget_);
-  m_mainWindow->m_copyEntry->plug(widget_);
-  m_mainWindow->m_deleteEntry->plug(widget_);
-  m_mainWindow->m_checkOutEntry->plug(widget_);
+void Controller::plugEntryActions(QPopupMenu* popup_) {
+  if(!popup_) {
+    return;
+  }
+
+//  m_mainWindow->m_newEntry->plug(popup_);
+  m_mainWindow->m_editEntry->plug(popup_);
+  m_mainWindow->m_copyEntry->plug(popup_);
+  m_mainWindow->m_deleteEntry->plug(popup_);
+  m_mainWindow->m_updateEntryMenu->plug(popup_);
+  // there's a bug in KActionMenu with KXMLGUIFactory::plugActionList
+  // pluging the menu action isn't enough to have the popup get populated
+  plugUpdateMenu(popup_);
+  popup_->insertSeparator();
+  m_mainWindow->m_checkOutEntry->plug(popup_);
+}
+
+void Controller::plugUpdateMenu(QPopupMenu* popup_) {
+  QPopupMenu* updatePopup = 0;
+  const uint count = popup_->count();
+  for(uint i = 0; i < count; ++i) {
+    QMenuItem* item = popup_->findItem(popup_->idAt(i));
+    if(item && item->text() == m_mainWindow->m_updateEntryMenu->text()) {
+      updatePopup = item->popup();
+      break;
+    }
+  }
+
+  if(!updatePopup) {
+    return;
+  }
+
+  // I can't figure out why the actions get duplicated, but they do
+  // so clear them all
+  m_mainWindow->m_updateAll->unplug(updatePopup);
+  for(QPtrListIterator<KAction> it(m_mainWindow->m_fetchActions); it.current(); ++it) {
+    it.current()->unplug(updatePopup);
+  }
+
+  // clear separator, too
+  updatePopup->clear();
+
+  m_mainWindow->m_updateAll->plug(updatePopup);
+  updatePopup->insertSeparator();
+  for(QPtrListIterator<KAction> it(m_mainWindow->m_fetchActions); it.current(); ++it) {
+    it.current()->plug(updatePopup);
+  }
 }
 
 void Controller::updateActions() const {
   bool emptySelection = m_selectedEntries.isEmpty();
-  m_mainWindow->m_editEntry->setEnabled(!emptySelection);
-  m_mainWindow->m_copyEntry->setEnabled(!emptySelection);
-  m_mainWindow->m_deleteEntry->setEnabled(!emptySelection);
-  m_mainWindow->m_checkOutEntry->setEnabled(!emptySelection);
+  m_mainWindow->stateChanged(QString::fromLatin1("empty_selection"),
+                             emptySelection ? KXMLGUIClient::StateNoReverse : KXMLGUIClient::StateReverse);
+  for(QPtrListIterator<KAction> it(m_mainWindow->m_fetchActions); it.current(); ++it) {
+    it.current()->setEnabled(!emptySelection);
+  }
+  //only enable citation items when it's a bibliography
+  bool isBibtex = Kernel::self()->collectionType() == Data::Collection::Bibtex;
+  if(isBibtex) {
+    m_mainWindow->action("cite_clipboard")->setEnabled(!emptySelection);
+    m_mainWindow->action("cite_lyxpipe")->setEnabled(!emptySelection);
+    m_mainWindow->action("cite_openoffice")->setEnabled(!emptySelection);
+  }
   m_mainWindow->m_checkInEntry->setEnabled(canCheckIn());
 
   if(m_selectedEntries.count() < 2) {
     m_mainWindow->m_editEntry->setText(i18n("&Edit Entry..."));
-    m_mainWindow->m_copyEntry->setText(i18n("&Copy Entry"));
+    m_mainWindow->m_copyEntry->setText(i18n("D&uplicate Entry"));
+    m_mainWindow->m_updateEntryMenu->setText(i18n("&Update Entry"));
     m_mainWindow->m_deleteEntry->setText(i18n("&Delete Entry"));
   } else {
     m_mainWindow->m_editEntry->setText(i18n("&Edit Entries..."));
-    m_mainWindow->m_copyEntry->setText(i18n("&Copy Entries"));
+    m_mainWindow->m_copyEntry->setText(i18n("D&uplicate Entries"));
+    m_mainWindow->m_updateEntryMenu->setText(i18n("&Update Entries"));
     m_mainWindow->m_deleteEntry->setText(i18n("&Delete Entries"));
   }
 }
 
-void Controller::addedBorrower(Data::Borrower* borrower_) {
+void Controller::addedBorrower(Data::BorrowerPtr borrower_) {
   m_mainWindow->addLoanView(); // just in case
   for(ObserverVec::Iterator it = m_observers.begin(); it != m_observers.end(); ++it) {
     it->addBorrower(borrower_);
@@ -505,14 +615,14 @@ void Controller::addedBorrower(Data::Borrower* borrower_) {
   m_mainWindow->m_viewTabs->setTabBarHidden(false);
 }
 
-void Controller::modifiedBorrower(Data::Borrower* borrower_) {
+void Controller::modifiedBorrower(Data::BorrowerPtr borrower_) {
   for(ObserverVec::Iterator it = m_observers.begin(); it != m_observers.end(); ++it) {
     it->modifyBorrower(borrower_);
   }
   hideTabs();
 }
 
-void Controller::addedFilter(Filter* filter_) {
+void Controller::addedFilter(FilterPtr filter_) {
   m_mainWindow->addFilterView(); // just in case
   for(ObserverVec::Iterator it = m_observers.begin(); it != m_observers.end(); ++it) {
     it->addFilter(filter_);
@@ -520,7 +630,7 @@ void Controller::addedFilter(Filter* filter_) {
   m_mainWindow->m_viewTabs->setTabBarHidden(false);
 }
 
-void Controller::removedFilter(Filter* filter_) {
+void Controller::removedFilter(FilterPtr filter_) {
   for(ObserverVec::Iterator it = m_observers.begin(); it != m_observers.end(); ++it) {
     it->removeFilter(filter_);
   }
@@ -535,7 +645,7 @@ void Controller::slotCheckOut() {
   Data::EntryVec loanedEntries = m_selectedEntries;
 
   // check to see if any of the entries are already on-loan, and warn user
-  QMap<QString, Data::Entry*> alreadyLoaned;
+  QMap<QString, Data::EntryPtr> alreadyLoaned;
   const Data::BorrowerVec& borrowers = Data::Document::self()->collection()->borrowers();
   for(Data::BorrowerVec::ConstIterator it = borrowers.begin(); it != borrowers.end(); ++it) {
     const Data::LoanVec& loans = it->loans();
@@ -552,8 +662,8 @@ void Controller::slotCheckOut() {
                                       "times. They will be removed from the list of items "
                                       "to lend."),
                                       alreadyLoaned.keys());
-    QMapConstIterator<QString, Data::Entry*> it = alreadyLoaned.constBegin();
-    QMapConstIterator<QString, Data::Entry*> end = alreadyLoaned.constEnd();
+    QMapConstIterator<QString, Data::EntryPtr> it = alreadyLoaned.constBegin();
+    QMapConstIterator<QString, Data::EntryPtr> end = alreadyLoaned.constEnd();
     for( ; it != end; ++it) {
       loanedEntries.remove(it.data());
     }
@@ -583,7 +693,7 @@ void Controller::slotCheckIn(const Data::EntryVec& entries_) {
     Data::BorrowerVec vec = Data::Document::self()->collection()->borrowers();
     // vec.end() must be in the loop, do NOT cache the value, it could change!
     for(Data::BorrowerVec::Iterator bIt = vec.begin(); bIt != vec.end(); ++bIt) {
-      Data::Loan* l = bIt->loan(it);
+      Data::LoanPtr l = bIt->loan(it.data());
       if(l) {
         loans.append(l);
         // assume it's only loaned once
@@ -616,4 +726,8 @@ bool Controller::canCheckIn() const {
   return false;
 }
 
+void Controller::updatedFetchers() {
+  m_mainWindow->updateEntrySources();
+}
+
 #include "controller.moc"
