@@ -1,5 +1,5 @@
 /***************************************************************************
-    copyright            : (C) 2001-2004 by Robby Stephenson
+    copyright            : (C) 2001-2006 by Robby Stephenson
     email                : robby@periapsis.org
  ***************************************************************************/
 
@@ -15,15 +15,12 @@
 #define DETAILEDLISTVIEW_H
 
 namespace Tellico {
-  namespace Data {
-    class Collection;
-  }
-  class Filter;
-  class EntryItem;
+  class DetailedEntryItem;
 }
 
-#include "multiselectionlistview.h"
-#include "entry.h"
+#include "gui/listview.h"
+#include "observer.h"
+#include "filter.h"
 
 #include <kpopupmenu.h>
 
@@ -31,7 +28,6 @@ namespace Tellico {
 #include <qstringlist.h>
 #include <qpixmap.h>
 #include <qvaluevector.h>
-#include <qguardedptr.h>
 
 namespace Tellico {
 
@@ -40,9 +36,8 @@ namespace Tellico {
  * collection.
  *
  * @author Robby Stephenson
- * @version $Id: detailedlistview.h 862 2004-09-15 01:49:51Z robby $
  */
-class DetailedListView : public MultiSelectionListView {
+class DetailedListView : public GUI::ListView, public Observer {
 Q_OBJECT
 
 public:
@@ -59,18 +54,14 @@ public:
    * Event filter used to popup the menu
    */
   bool eventFilter(QObject* obj, QEvent* ev);
-  /**
-   * Clears the selection.
-   */
-  void clearSelection();
   /**
    * Selects the item which refers to a certain entry.
    *
    * @param entry A pointer to the entry
    */
-  void setEntrySelected(Data::Entry* entry);
-  void setFilter(const Filter* filter);
-  const Filter* filter() const { return m_filter; }
+  void setEntrySelected(Data::EntryPtr entry);
+  void setFilter(FilterPtr filter);
+  FilterPtr filter() { return m_filter; }
 
   int prevSortedColumn() const;
   int prev2SortedColumn() const;
@@ -80,7 +71,7 @@ public:
   QString sortColumnTitle2() const;
   QString sortColumnTitle3() const;
   QStringList visibleColumns() const;
-  Data::EntryList visibleEntries();
+  Data::EntryVec visibleEntries();
   /**
    * Returns whether the given column is formatted as a number or not.
    *
@@ -91,36 +82,42 @@ public:
   /**
    * @param coll A pointer to the collection
    */
-  void addCollection(Tellico::Data::Collection* coll);
+  void addCollection(Data::CollPtr coll);
   /**
    * Removes all items which refers to a entry within a collection.
    *
    * @param coll A pointer to the collection
    */
-  void removeCollection(Tellico::Data::Collection* coll);
+  void removeCollection(Data::CollPtr coll);
+
   /**
    * Adds a new list item showing the details for a entry.
    *
    * @param entry A pointer to the entry
    */
-  void addEntry(Data::Entry* entry);
+  virtual void addEntries(Data::EntryVec entries);
   /**
    * Modifies any item which refers to a entry, resetting the column contents.
    *
    * @param entry A pointer to the entry
    */
-  void modifyEntry(Data::Entry* entry);
+  virtual void modifyEntries(Data::EntryVec entries);
   /**
    * Removes any item which refers to a certain entry.
    *
    * @param entry A pointer to the entry
    */
-  void removeEntry(Data::Entry* entry);
-  void addField(Data::Field* field, int width);
-  void modifyField(Data::Field* newField, Data::Field* oldField);
-  void removeField(Data::Field* field);
-  void reorderFields(const Data::FieldList& list);
-  void saveConfig(Tellico::Data::Collection* coll);
+  virtual void removeEntries(Data::EntryVec entries);
+
+  virtual void addField(Data::CollPtr, Data::FieldPtr field);
+  void addField(Data::FieldPtr field, int width);
+  virtual void modifyField(Data::CollPtr, Data::FieldPtr oldField, Data::FieldPtr newField);
+  virtual void removeField(Data::CollPtr, Data::FieldPtr field);
+
+  void reorderFields(const Data::FieldVec& fields);
+  /**
+   * saveConfig is only needed for custom collections */
+  void saveConfig(Data::CollPtr coll, int saveConfig);
   /**
    * Select all visible items.
    */
@@ -133,6 +130,7 @@ public:
    * @param height Height
    */
   void setPixmapSize(int width, int height) { m_pixWidth = width; m_pixHeight = height; }
+  void resetEntryStatus();
 
 public slots:
   /**
@@ -144,7 +142,8 @@ public slots:
    */
   void slotRefresh();
 
-protected:
+private:
+  DetailedEntryItem* addEntryInternal(Data::EntryPtr entry);
   /**
    * A helper method to populate an item. The column text is initialized by pulling
    * the contents from the entry pointer of the item, so it should properly be set
@@ -152,9 +151,9 @@ protected:
    *
    * @param item A pointer to the item
    */
-  void populateItem(EntryItem* item);
+  void populateItem(DetailedEntryItem* item);
   void populateColumn(int col);
-  void setPixmapAndText(EntryItem* item, int col, Data::Field* field);
+  void setPixmapAndText(DetailedEntryItem* item, int col, Data::FieldPtr field);
 
   /**
    * A helper method to locate any item which refers to a certain entry. If none
@@ -163,12 +162,12 @@ protected:
    * @param entry A pointer to the entry
    * @return A pointer to the item
    */
-  EntryItem* const locateItem(const Data::Entry* entry);
+  DetailedEntryItem* const locateItem(Data::EntryPtr entry);
   void showColumn(int col);
   void hideColumn(int col);
   void updateFirstSection();
 
-protected slots:
+private slots:
   /**
    * Handles the appearance of the popup menu.
    *
@@ -187,31 +186,26 @@ protected slots:
    * Slot to update the position of the pixmap
    */
   void slotUpdatePixmap();
-  void slotDoubleClicked(QListViewItem* item);
-
-signals:
-  /**
-   * Signals a desire to delete a entry.
-   *
-   * @param entry A pointer to the entry
-   */
-  void signalDeleteEntry(Tellico::Data::Entry* entry);
-  /**
-   * Signals that a fraction of an operation has been completed.
-   *
-   * @param f The fraction, 0 =< f >= 1
-   */
-  void signalFractionDone(float f);
 
 private:
-  KPopupMenu* m_itemMenu;
+  struct ConfigInfo {
+    QStringList cols;
+    QValueList<int> widths;
+    QValueList<int> order;
+    int colSorted;
+    bool ascSort : 1;
+    int prevSort;
+    int prev2Sort;
+  };
+
   KPopupMenu* m_headerMenu;
   QValueVector<int> m_columnWidths;
   QValueVector<bool> m_isNumber;
   QValueVector<bool> m_isDirty;
   QPixmap m_entryPix;
   QPixmap m_checkPix;
-  const Filter* m_filter;
+
+  FilterPtr m_filter;
   int m_prevSortColumn;
   int m_prev2SortColumn;
   int m_firstSection;
