@@ -154,7 +154,6 @@ public:
   bool loadImage(const QString& id);
   bool loadAllImagesNow() const;
   bool allImagesOnDisk() const { return m_allImagesOnDisk; }
-  void setAllImagesOnDisk(bool b) { m_allImagesOnDisk = b; }
   int imageCount() const;
   EntryVec filteredEntries(Filter::Ptr filter) const;
   /**
@@ -167,7 +166,11 @@ public:
   void checkInEntry(EntryPtr entry);
   void checkOutEntry(EntryPtr entry);
 
-  void removeImagesNotInCollection(Data::EntryVec entries);
+  /**
+   * The second entry vector contains entries with images which should not be removed
+   * in addition to those already in the collection
+   */
+  void removeImagesNotInCollection(EntryVec entries, EntryVec entriesToKeep);
 
 public slots:
   /**
