@@ -36,7 +36,6 @@ typedef KSortableItem<Type, QString> TypePair; // fetcher info, type and name of
 typedef KSortableValueList<Type, QString> TypePairList;
 typedef QMap<FetchKey, QString> KeyMap; // map key type to name of key
 typedef Vector<Fetcher> FetcherVec;
-typedef Vector<const Fetcher> CFetcherVec;
 
 /**
  * A manager for handling all the different classes of Fetcher.
@@ -51,14 +50,15 @@ public:
 
   ~Manager();
 
-  TypePairList sources() const;
   KeyMap keyMap(const QString& source = QString::null) const;
   void startSearch(const QString& source, FetchKey key, const QString& value);
+  void continueSearch();
   void stop();
   bool canFetch() const;
+  bool hasMoreResults() const;
   void loadFetchers();
   const FetcherVec& fetchers() const { return m_fetchers; }
-  CFetcherVec fetchers(int type) const;
+  FetcherVec fetchers(int type);
   TypePairList typeList();
   ConfigWidget* configWidget(QWidget* parent, Type type, const QString& name);
 
@@ -91,6 +91,7 @@ private:
   static bool bundledScriptHasExecPath(const QString& specFile, KConfig* config);
 
   FetcherVec m_fetchers;
+  int m_currentFetcherIndex;
   KeyMap m_keyMap;
   typedef QMap<Fetcher::Ptr, QString> ConfigMap;
   ConfigMap m_configMap;
