@@ -1,5 +1,5 @@
 /***************************************************************************
-    copyright            : (C) 2003-2005 by Robby Stephenson
+    copyright            : (C) 2003-2006 by Robby Stephenson
     email                : robby@periapsis.org
  ***************************************************************************/
 
@@ -16,13 +16,17 @@
 
 class KURL;
 class KSaveFile;
+class KFileItem;
+namespace KIO {
+  class Job;
+}
 
-class QString;
 class QDomDocument;
 class QFile;
 
 #include <qstring.h>
 #include <qcstring.h> // needed for QByteArray
+#include <qptrlist.h>
 
 namespace Tellico {
   class ImageFactory;
@@ -38,7 +42,6 @@ namespace Tellico {
 class FileHandler {
 
 friend class ImageFactory;
-//friend class Data::Image;
 
 public:
   /**
@@ -60,11 +63,11 @@ public:
     QString m_filename;
     bool m_isValid;
   };
-  friend class FileRef; // gcc 2.95 needs this
+  friend class FileRef;
 
   /**
    * Creates a FileRef for a given url. It's not meant to be used by methods in the class,
-   * Rather by a class wanted direct access to a file. The caller takes ownership of the pointer.
+   * Rather by a class wanting direct access to a file. The caller takes ownership of the pointer.
    *
    * @param url The url
    * @param quiet Whether error messages should be shown
@@ -75,6 +78,7 @@ public:
    * Read contents of a file into a string.
    *
    * @param url The URL of the file
+   * @param quiet whether the importer should report errors or not
    * @return A string containing the contents of a file
    */
   static QString readTextFile(const KURL& url, bool quiet=false);
@@ -106,7 +110,7 @@ public:
    * @param force Whether to force the write
    * @return A boolean indicating success
    */
-  static bool writeTextURL(const KURL& url, const QString& text, bool encodeUTF8, bool force=false);
+  static bool writeTextURL(const KURL& url, const QString& text, bool encodeUTF8, bool force=false, bool quiet=false);
   /**
    * Writes data to a url. If the file already exists, a "~" is appended
    * and the existing file is moved. If the file is remote, a temporary file is written and
@@ -117,9 +121,21 @@ public:
    * @param force Whether to force the write
    * @return A boolean indicating success
    */
-  static bool writeDataURL(const KURL& url, const QByteArray& data, bool force=false);
+  static bool writeDataURL(const KURL& url, const QByteArray& data, bool force=false, bool quiet=false);
+  /**
+   * Checks to see if a URL exists already, and if so, queries the user.
+   *
+   * @param url The target URL
+   * @return True if it is ok to continue, false otherwise.
+   */
+  static bool queryExists(const KURL& url);
+  static void clean();
 
 private:
+  class ItemDeleter;
+  friend class ItemDeleter;
+  static QPtrList<ItemDeleter> s_deleterList;
+
   /**
    * Read contents of a file into an image. It's private since everything should use the
    * ImageFactory methods.
@@ -129,13 +145,7 @@ private:
    * @return The image
    */
   static Data::Image* readImageFile(const KURL& url, bool quiet=false);
-  /**
-   * Checks to see if a URL exists already, and if so, queries the user.
-   *
-   * @param url The target URL
-   * @return True if it is ok to continue, false otherwise.
-   */
-  static bool queryExists(const KURL& url);
+  static Data::Image* readImageFile(const KURL& url, bool quiet, const KURL& referrer);
   /**
    * Writes the contents of a string to a file.
    *
